﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using DlhSoft.Windows.Data;
using Pert = DlhSoft.Windows.Controls.Pert;

namespace ProjectManagementControlsSample
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
            SetBaseline();
        }

        public static Schedule CustomSchedule
        {
            get
            {
                return new Schedule("Mon-Fri: 6h; Sat: 4h",
                    delegate(Date date)
                    {
                        if (date.DayOfWeek >= DayOfWeek.Monday && date.DayOfWeek <= DayOfWeek.Friday)
                        {
                            return new TimeIntervalCollection
                            {
                                new TimeInterval(date.Add(TimeSpan.Parse("08:00:00")), date.Add(TimeSpan.Parse("12:00:00"))),
                                new TimeInterval(date.Add(TimeSpan.Parse("13:00:00")), date.Add(TimeSpan.Parse("15:00:00")))
                            };
                        }
                        else if (date.DayOfWeek == DayOfWeek.Saturday)
                        {
                            return new TimeIntervalCollection
                            {
                                new TimeInterval(date.Add(TimeSpan.Parse("08:00:00")), date.Add(TimeSpan.Parse("12:00:00")))
                            };
                        }
                        return null;
                    });
            }
        }

        // Baseline information (estimated values).
        private void SetBaseline()
        {
            var task9 = GanttChartTaskListView.Items[9] as TaskItem;
            DlhSoft.Windows.Controls.GanttChartView.SetBaselineStart(task9, DateTime.Today.Add(TimeSpan.Parse("08:00:00")));
            DlhSoft.Windows.Controls.GanttChartView.SetBaselineFinish(task9, DateTime.Today.AddDays(3).Add(TimeSpan.Parse("16:00:00")));
            var task10 = GanttChartTaskListView.Items[10] as TaskItem;
            DlhSoft.Windows.Controls.GanttChartView.SetBaselineStart(task10, DateTime.Today.AddDays(4).Add(TimeSpan.Parse("08:00:00")));
            DlhSoft.Windows.Controls.GanttChartView.SetBaselineFinish(task10, DateTime.Today.AddDays(8).Add(TimeSpan.Parse("12:00:00")));
            var task14 = GanttChartTaskListView.Items[14] as TaskItem;
            DlhSoft.Windows.Controls.GanttChartView.SetBaselineStart(task14, DateTime.Today.AddDays(1).Add(TimeSpan.Parse("12:00:00")));
        }

        // PERT integration.
        private void PertChartViewTabItem_Selected(object sender, RoutedEventArgs e)
        {
            PertChartView.Items = GanttChartTaskListView.GetPertChartItems();
        }
        private void PertChartViewTabItem_Unselected(object sender, RoutedEventArgs e)
        {
            PertChartViewHighlightCriticalPathCheckBox.IsChecked = false;
            GanttChartTaskListView.DisposePertChartItems(PertChartView.Items);
        }
        private void PertChartViewHighlightCriticalPathCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            foreach (Pert.PertChartItem item in PertChartView.ManagedItems)
            {
                SetCriticalPathHighlighting(item, false);
                if (item.Predecessors != null)
                {
                    foreach (Pert.PredecessorItem predecessorItem in item.Predecessors)
                        SetCriticalPathHighlighting(predecessorItem, false);
                }
            }
            foreach (Pert.PertChartItem item in PertChartView.GetCriticalItems())
                SetCriticalPathHighlighting(item, true);
            foreach (Pert.PredecessorItem predecessorItem in PertChartView.GetCriticalDependencies())
                SetCriticalPathHighlighting(predecessorItem, true);
        }
        private void PertChartViewHighlightCriticalPathCheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            foreach (Pert.PredecessorItem predecessorItem in PertChartView.GetCriticalDependencies())
                SetCriticalPathHighlighting(predecessorItem, false);
            foreach (Pert.PertChartItem item in PertChartView.GetCriticalItems())
                SetCriticalPathHighlighting(item, false);
        }
        private void SetCriticalPathHighlighting(Pert.PertChartItem item, bool isHighlighted)
        {
            Pert.PertChartView.SetShapeStroke(item, isHighlighted ? Resources["PertChartRedBrush"] as Brush : PertChartView.ShapeStroke);
            Pert.PertChartView.SetTextForeground(item, isHighlighted ? Resources["PertChartRedBrush"] as Brush : PertChartView.TextForeground);
        }
        private void SetCriticalPathHighlighting(Pert.PredecessorItem predecessorItem, bool isHighlighted)
        {
            Pert.PertChartView.SetDependencyLineStroke(predecessorItem, isHighlighted ? Resources["PertChartRedBrush"] as Brush : PertChartView.DependencyLineStroke);
            Pert.PertChartView.SetDependencyTextForeground(predecessorItem, isHighlighted ? Resources["PertChartRedBrush"] as Brush : PertChartView.DependencyTextForeground);
        }
        private void PertChartViewPrint_Click(object sender, RoutedEventArgs e)
        {
            PertChartView.Print();
        }
        private void NetworkDiagramViewTabItem_Selected(object sender, RoutedEventArgs e)
        {
            NetworkDiagramView.Items = GanttChartTaskListView.GetNetworkDiagramItems();
            // Optionally, reposition start and finish milestones between the first and second rows of the view.
            Dispatcher.BeginInvoke((Action)delegate { NetworkDiagramView.RepositionEnds(); });
        }
        private void NetworkDiagramViewTabItem_Unselected(object sender, RoutedEventArgs e)
        {
            NetworkDiagramViewHighlightCriticalPathCheckBox.IsChecked = false;
            GanttChartTaskListView.DisposeNetworkDiagramItems(NetworkDiagramView.Items);
        }
        private void NetworkDiagramViewHighlightCriticalPathCheckBox_Checked(object sender, RoutedEventArgs e)
        {
            foreach (Pert.NetworkDiagramItem item in NetworkDiagramView.ManagedItems)
            {
                SetCriticalPathHighlighting(item, false);
                if (item.Predecessors != null)
                {
                    foreach (Pert.NetworkPredecessorItem predecessorItem in item.Predecessors)
                        SetCriticalPathHighlighting(predecessorItem, false);
                }
            }
            foreach (Pert.NetworkDiagramItem item in NetworkDiagramView.GetCriticalItems())
                SetCriticalPathHighlighting(item, true);
            foreach (Pert.NetworkPredecessorItem predecessorItem in NetworkDiagramView.GetCriticalDependencies())
                SetCriticalPathHighlighting(predecessorItem, true);
        }
        private void NetworkDiagramViewHighlightCriticalPathCheckBox_Unchecked(object sender, RoutedEventArgs e)
        {
            foreach (Pert.NetworkPredecessorItem predecessorItem in NetworkDiagramView.GetCriticalDependencies())
                SetCriticalPathHighlighting(predecessorItem, false);
            foreach (Pert.NetworkDiagramItem item in NetworkDiagramView.GetCriticalItems())
                SetCriticalPathHighlighting(item, false);
        }
        private void SetCriticalPathHighlighting(Pert.NetworkDiagramItem item, bool isHighlighted)
        {
            Pert.NetworkDiagramView.SetShapeStroke(item, isHighlighted ? Resources["NetworkDiagramRedBrush"] as Brush : NetworkDiagramView.ShapeStroke);
        }
        private void SetCriticalPathHighlighting(Pert.NetworkPredecessorItem predecessorItem, bool isHighlighted)
        {
            Pert.NetworkDiagramView.SetDependencyLineStroke(predecessorItem, isHighlighted ? Resources["NetworkDiagramRedBrush"] as Brush : NetworkDiagramView.DependencyLineStroke);
        }
        private void NetworkDiagramViewPrint_Click(object sender, RoutedEventArgs e)
        {
            NetworkDiagramView.Print();
        }
    }
}

